--[[
-- Copyright 2018 AKA Russell
--
-- This file is part of Scarlet.
--
-- Scarlet is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- Scarlet is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with Scarlet.  If not, see <http://www.gnu.org/licenses/>.
-- ]]
-- [[
-- This module contains generic utilities that may prove useful to various
-- combat abilities, such as resolving a target based on a Love KeyConstant.
-- ]]
local M = {}

local combatstateops = require 'combat/combatstate'

--[[
-- Resolves a Love KeyConstant into a target, returns nil if the key does
-- not represent a valid target
--
-- combatstate: The current state of combat
-- targetkey: A Love KeyConstant representing a target. This is expected to
-- be either a digit (in string form) or a function key (i.e. "f12"). If the
-- targetkey is a digit, then it returns the target from combatstate.enemies,
-- if it is a function key then it returns the target from combatstate.allies
--
-- return The target represented by the passed in targetkey
--]]
function M.resolvetargetkey(combatstate, targetkey)
    local target
    if LOVE and string.find(targetkey, "f") then
        local characternumber = tonumber(string.sub(targetkey, 2))
        target = characternumber and combatstate.allies[characternumber]
    elseif LOVE then
        local characternumber = tonumber(targetkey)
        target = characternumber and combatstate.enemies[characternumber]
    else
        local characternumber = tonumber(targetkey)
        target = characternumber and combatstate.characters[characternumber]
    end
    return target
end

--[[
-- Given a combatstate and combat primitive, returns the user
-- and target characters of the primitive.
--
-- combatstate: The current state of combat
-- primitive: The combat primitive whose targets need to be resolved
--
-- returns The user and the target of the primitive
--]]
function M.resolveusertarget(combatstate, primitive)
    local user = combatstateops.findcharacter(combatstate, primitive.username)
    local targetname = primitive.targetname
    local target = combatstateops.findcharacter(combatstate, targetname)
    return user, target
end

return M
