# Overview #

Powers are special abilities that allow characters to manipulate the rules of
combat. A small sampling of what they can do:

1. Increase/Reduce damage done.
2. Increase the number of attacks a character has.
3. Do additional damage.
4. Redirect damage.

Typically, these powers will either cost energy (i.e. health) or increase
attrition (the amount of energy lost at the end of each round). At the same
time, they will be absolutely critical to winning. So combat is a careful
balancing act of which powers to use when.

# Structure #

Powers are standard combat pages (meaning it has functions
`body`, `choices,` `automatics` and operation `transformer` on combatstate)
along with some additional metadata.

## AI metadata ##

This meta data is intended to be  used by AI functions of various
intelligence. These will generally only be used by the AI to select a power,
and so are not necessary for powers used only by PC's.
1. `damage` - One of the following integers: -1, 0, 1. If -1, the power heals.
    If 0, the power neither heals nor does damage, if 1 the power does damage.
2. `targets` - Only necessary if the function inflicts damage. A function on
   `combatstate` that returns the list of characters who can be targeted by
    this power. Most often, this will return either the user's allies or enemies,
    but it can be used for more intelligent targeting as well. For example, it
    can ensure that the user doesn't attempt to inflict a status on someone who
    already has that status.
3. `aiexecution` - Only necessary for powers that inflict damage. A function
    that takes the combatstate, a previous page, and a variable number of Love
    KeyConstants representing the selected targets, and returns a combat page
    that executes the power. Note that the targets are only a suggestion.
    An implementor may choose to ignore the selected targets in favor of their
    own target selection.

The `aiexecution` function is used by the average AI after it has selected its
targets. The average AI only uses powers that inflict damage. Therefore, for
powers that don't, each power can select its targets directly when resolving
into combat primitives. Similarly, the `targets` function is used by the
average AI to decide which pool of characters to select targets from.

## Metadata ##

These parameters are used by the AI, and displayed to the player when they
select the power, so defining them even for PC powers is highly recommended.

1. `cost` - Either a number, or a function on character that returns a number
   describing how much energy it costs to use the power.
2. `numtargets` - The number of targets this skill can have. A -1 means the
    skill targets *everyone* everytime. A -2 means the skill targets all opponents,
    while a -3 means the skill targets all allies.
3. `stats` - A list of the statistic names that influence the potency of this
    skill in decreasing influence.
6. `description` - Either a brief English description of the power that will
    be displayed to the user when selecting the power, or a function on
    character that returns a description to be displayed to the user.
